from typing import Any, Dict, Optional
import aiohttp
import logging
from app.services.external_api.base_external_service import BaseExternalService
from app.services.external_api.tcgplayer.tcgplayer_credentials import TCGPlayerCredentials

logger = logging.getLogger(__name__)

class BaseTCGPlayerService(BaseExternalService):
    def __init__(self):
        super().__init__(
            store_base_url="https://store.tcgplayer.com",
            login_endpoint="/oauth/login",
            pricing_endpoint="/Admin/Pricing",
            staged_inventory_endpoint=self.pricing_endpoint + "/DownloadStagedInventoryExportCSV?type=Pricing",
            live_inventory_endpoint=self.pricing_endpoint + "/DownloadMyExportCSV?type=Pricing"
        )
        self.credentials = TCGPlayerCredentials()

    def _get_headers(self, method: str) -> Dict[str, str]:
        """Get headers based on the HTTP method"""
        base_headers = {
            'accept': 'text/html,application/xhtml+xml,application/xml;q=0.9,image/avif,image/webp,image/apng,*/*;q=0.8',
            'accept-language': 'en-US,en;q=0.8',
            'priority': 'u=0, i',
            'referer': 'https://store.tcgplayer.com/admin/pricing',
            'sec-ch-ua': '"Not A(Brand";v="8", "Chromium";v="132", "Brave";v="132"',
            'sec-ch-ua-mobile': '?0', 
            'sec-ch-ua-platform': '"macOS"',
            'sec-fetch-dest': 'document',
            'sec-fetch-mode': 'navigate',
            'sec-fetch-site': 'same-origin',
            'sec-fetch-user': '?1',
            'sec-gpc': '1',
            'upgrade-insecure-requests': '1',
            'user-agent': 'Mozilla/5.0 (Macintosh; Intel Mac OS X 10_15_7) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/132.0.0.0 Safari/537.36'
        }
        
        if method == 'POST':
            post_headers = {
                'cache-control': 'max-age=0',
                'content-type': 'application/x-www-form-urlencoded',
                'origin': 'https://store.tcgplayer.com'
            }
            base_headers.update(post_headers)
            
        return base_headers

    async def _make_request(
        self,
        method: str,
        endpoint: str,
        params: Optional[Dict[str, Any]] = None,
        headers: Optional[Dict[str, str]] = None,
        data: Optional[Dict[str, Any]] = None,
        auth_required: bool = True
    ) -> Dict[str, Any]:
        session = await self._get_session()
        url = f"{self.store_base_url}{endpoint}"
        
        # Get the authentication cookie if required
        if auth_required:
            cookie = self.credentials.get_cookie()
            if not cookie:
                raise RuntimeError("TCGPlayer authentication cookie not set. Please set the cookie using TCGPlayerCredentials.set_cookie()")
            
            # Get method-specific headers and update with any provided headers
            request_headers = self._get_headers(method)
            if headers:
                request_headers.update(headers)
            request_headers["Cookie"] = cookie
        else:
            request_headers = headers or {}
        
        try:
            async with session.request(method, url, params=params, headers=request_headers, json=data) as response:
                if response.status == 401:
                    raise RuntimeError("TCGPlayer authentication failed. Cookie may be invalid or expired.")
                response.raise_for_status()
                return await response.json()
        except aiohttp.ClientError as e:
            logger.error(f"TCGPlayer API request failed: {str(e)}")
            raise
        except Exception as e:
            logger.error(f"Unexpected error during TCGPlayer API request: {str(e)}")
            raise

    async def _get_session(self) -> aiohttp.ClientSession:
        if self.session is None or self.session.closed:
            self.session = aiohttp.ClientSession()
        return self.session

    async def close(self):
        if self.session and not self.session.closed:
            await self.session.close() 