from typing import Dict, List, Optional
from datetime import datetime
from app.services.external_api.tcgplayer.base_tcgplayer_service import BaseTCGPlayerService

class TCGPlayerOrderService(BaseTCGPlayerService):
    def __init__(self):
        super().__init__()

    async def get_orders(
        self,
        status: Optional[str] = None,
        start_date: Optional[datetime] = None,
        end_date: Optional[datetime] = None,
        limit: int = 100
    ) -> List[Dict]:
        """
        Get a list of orders with optional filtering
        
        Args:
            status: Filter by order status (e.g., "Shipped", "Processing")
            start_date: Filter orders after this date
            end_date: Filter orders before this date
            limit: Maximum number of orders to return
            
        Returns:
            List of orders matching the criteria
        """
        endpoint = "/orders"
        params = {"limit": limit}
        
        if status:
            params["status"] = status
        if start_date:
            params["startDate"] = start_date.isoformat()
        if end_date:
            params["endDate"] = end_date.isoformat()
            
        response = await self._make_request("GET", endpoint, params=params)
        return response.get("results", [])

    async def get_order_details(self, order_id: str) -> Dict:
        """
        Get detailed information about a specific order
        
        Args:
            order_id: TCGPlayer order ID
            
        Returns:
            Detailed order information
        """
        endpoint = f"/orders/{order_id}"
        response = await self._make_request("GET", endpoint)
        return response

    async def get_order_items(self, order_id: str) -> List[Dict]:
        """
        Get items in a specific order
        
        Args:
            order_id: TCGPlayer order ID
            
        Returns:
            List of items in the order
        """
        endpoint = f"/orders/{order_id}/items"
        response = await self._make_request("GET", endpoint)
        return response.get("results", [])

    async def get_order_status(self, order_id: str) -> Dict:
        """
        Get the current status of an order
        
        Args:
            order_id: TCGPlayer order ID
            
        Returns:
            Order status information
        """
        endpoint = f"/orders/{order_id}/status"
        response = await self._make_request("GET", endpoint)
        return response

    async def update_order_status(
        self,
        order_id: str,
        status: str,
        tracking_number: Optional[str] = None
    ) -> Dict:
        """
        Update the status of an order
        
        Args:
            order_id: TCGPlayer order ID
            status: New status for the order
            tracking_number: Optional tracking number for shipping
            
        Returns:
            Updated order information
        """
        endpoint = f"/orders/{order_id}/status"
        data = {"status": status}
        if tracking_number:
            data["trackingNumber"] = tracking_number
            
        response = await self._make_request("PUT", endpoint, data=data)
        return response
